@@ -1,12 +1,18 @@
 # encoding=utf-8
-import argparse
 import gettext
 import logging
 import os
+import sys
 
 import wpull.version
 
 
+if sys.version_info >= (2, 7):
+    import argparse
+else:
+    from wpull.backport import argparse
+
+
 _ = gettext.gettext
 
 
@@ -54,6 +60,11 @@ class AppArgumentParser(argparse.ArgumentParser):
         items = list([item.strip() for item in items])
         return items
 
+    def parse_args(self, args=None, namespace=None):
+        args = super().parse_args(args=args, namespace=namespace)
+        self._post_parse_args(args)
+        return args
+
     def _add_app_args(self):
         self.add_argument(
             'urls',
@@ -225,11 +236,11 @@ class AppArgumentParser(argparse.ArgumentParser):
             const='disable',
             help=_('don’t use anti-clobbering filenames'),
         )
-        self.add_argument(
+        group.add_argument(
             '-c',
             '--continue',
             action='store_true',
-            dest='continue_file',
+            dest='continue_download',
             help=_('resume downloading a partially-downloaded file'),
         )
 #         self.add_argument(
@@ -241,8 +252,7 @@ class AppArgumentParser(argparse.ArgumentParser):
         clobber_group.add_argument(
             '-N',
             '--timestamping',
-            action='store_const',
-            const='timestamping',
+            action='store_true',
             help=_('only download files that are newer than local files'),
         )
         group.add_argument(
@@ -336,11 +346,11 @@ class AppArgumentParser(argparse.ArgumentParser):
 #             action='store_true',
 #             help=_('disable caching of DNS lookups'),
 #         )
-#         self.add_argument(
-#             '--rotate-dns',
-#             action='store_true',
-#             help=_('use different resolved IP addresses on requests'),
-#         )
+        group.add_argument(
+            '--rotate-dns',
+            action='store_true',
+            help=_('use different resolved IP addresses on requests'),
+        )
 #         self.add_argument(
 #             '--restrict-file-names',
 #             metavar='OS',
@@ -518,9 +528,13 @@ class AppArgumentParser(argparse.ArgumentParser):
             default=True,
             help=_('ignore robots.txt directives'),
         )
-#         self.add_argument(
-#             '--no-http-keep-alive',
-#         )
+        group.add_argument(
+            '--no-http-keep-alive',
+            dest='http_keep_alive',
+            action='store_false',
+            default=True,
+            help=_('disable persistent HTTP connections')
+        )
 #         self.add_argument(
 #             '--no-cookies'
 #         )
@@ -845,3 +859,17 @@ class AppArgumentParser(argparse.ArgumentParser):
             action='store_true',
             help=_('don’t follow to parent directories on URL path'),
         )
+
+    def _post_parse_args(self, args):
+        if args.warc_file:
+            self._post_warc_args(args)
+
+    def _post_warc_args(self, args):
+        option_names = ('clobber_method', 'timestamping', 'continue_download')
+
+        for option_name in option_names:
+            if vars(args).get(option_name):
+                self.error(
+                    _('WARC output cannot be combined with {option_name}.') \
+                        .format(option_name=option_name)
+                )
